@@ -8,7 +8,8 @@ def find_path ():
 	(head, tail) = split (path)
 	if tail == 'site-packages':
 	    base = basename (head)
-	    print base + '/' + tail
+	    print path
+	    #print base + '/' + tail
 	    return
 
 find_path ()
