@@ -28,6 +28,7 @@
 import re
 import os
 import sys
+import time
 
 try:
     import pyatspi as atspi, Accessibility
@@ -35,6 +36,8 @@ except ImportError:
     import atspi
     import Accessibility
 
+from ldtplib.ldtplibutils import *
+
 ldtpDebug = os.getenv ('LDTP_DEBUG') # Enable debugging
 
 class objInst:
@@ -113,6 +116,53 @@ class objInst:
         self.unknown = 0
         self.embeddedComponent = 0
 
+class LdtpClientContext:
+    def __init__ (self, localeLang, windowName, appHandleaccessible, appmap, request, response, guiHandle):
+        self.localeLang = localeLang
+        self.windowName = windowName
+        self.appHandle  = appHandleaccessible
+        self.appmap     = appmap
+        self.request    = request
+        self.response   = response
+        self.guiHandle  = guiHandle
+
+class LdtpGuiHandle:
+    def __init__ (self, handle, classId):
+        self.handle  = handle
+        self.classId = classId
+
+class Packet:
+    def __init__ (self, packet, length):
+        self.packet = packet
+        self.length = length
+
+class ParentPathInfo:
+    def __init__ (self, childIndex, count):
+        self.childIndex = childIndex
+        self.count      = count
+
+class Node:
+    def __init__ (self, childIndex, node):
+        self.childIndex = childIndex
+        self.node = node
+
+class LdtpRequest:
+    def __init__ (self, requestType, command, application, requestId, context, component, actionName, argsList):
+        self.requestType = requestType
+        self.command     = command
+        self.application = application
+        self.requestId   = requestId
+        self.context     = context
+        self.component   = component
+        self.actionName  = actionName
+        self.argsList    = argsList
+
+class LdtpResponse:
+    def __init__ (self, status, data, length):
+        self.status = status
+        self.data   = data
+        self.length = length
+
 class SearchObjInfo:
     def __init__ (self, key, objIsWindow = False, pattern = None):
         self.key = key
@@ -175,9 +225,10 @@ class keyboard:
                         75 : '<F9>', 76 : '<F10>', 95 : '<F11>', 96 : '<F12>'}
 
 class appmap:
-    def __init__ (self, objectInst):
+    def __init__ (self, objectInst,  registry):
         self.hashmap = None
         self.hierarchy = ''
+        self.registry    = registry
         self.objectInst = objectInst
         self.applicationmap = window ('list_of_windows', None, None).children
 
@@ -356,7 +407,7 @@ class appmap:
             num += 1
         return name + str (num)
 
-    def dumpTree (self, accessible, parentName = None):
+    def dumpTree (self, accessible, parentName = None,  forceReScan = False):
         role = accessible.getRole ()
         if role == Accessibility.ROLE_TABLE_CELL or \
                role == Accessibility.ROLE_SEPARATOR or \
@@ -371,7 +422,6 @@ class appmap:
             if relationNames is not None and relationNames != '':
                 name = relationNames
                 label = 'label_by'
-        global ldtpDebug
         #if ldtpDebug and ldtpDebug == '2':
         #    print '%s* %s "%s": %s' % ('\t', roleName, name, description)
         objectInfo = self.getObjectInfo (accessible, parentName)
@@ -382,29 +432,33 @@ class appmap:
                    role == Accessibility.ROLE_FILE_CHOOSER or role == Accessibility.ROLE_WINDOW:
                 flag = True
             if name != 'unnamed' and re.sub ("(?i) +", "", name) != '':
-                strippedStr = self.escapeChars (name, False)
+                strippedStr = escapeChars (name, False)
                 own_name = objectInfo.prefix + strippedStr
                 if flag:
-                    if self.applicationmap != {} and own_name in self.applicationmap:
+                    if self.applicationmap != {} and own_name in self.applicationmap and forceReScan == False:
                         if ldtpDebug:
                             print 'has_key', own_name
                         # Since the window info is already in appmap, don't process further
                         return
                     cur_win = window (objectInfo.objType, name, parentName, accessible.getIndexInParent ())
+                    if ldtpDebug:
+                        print 'Window:',  own_name,  cur_win,  type (own_name),  type (cur_win),  own_name in self.applicationmap,  forceReScan
+                    if own_name in self.applicationmap and forceReScan:
+                        del self.applicationmap [own_name]
+                    if ldtpDebug:
+                        print 'Window:',  own_name,  cur_win,  type (own_name),  type (cur_win),  own_name in self.applicationmap,  forceReScan
                     self.applicationmap [own_name] = cur_win
                     self.hashmap = cur_win.children
                 #else:
                 #    if self.hashmap and own_name in self.hashmap:
                 #        own_name = self.get_non_conflicting_name (own_name)
                 #    if ldtpDebug and self.hashmap:
-                #        # TODO: Comment out the below line after Debugging
                 #        assert own_name not in self.hashmap, ('Name Conflict in Hashmap. ' + \
                 #                                              own_name + 'already present\n' + foo)
                 #    self.hashmap [own_name] = widget (objectInfo.objType, name, parentName, accessible.getIndexInParent ())
                 if self.hashmap and own_name in self.hashmap:
                     own_name = self.get_non_conflicting_name (own_name)
                 if ldtpDebug and self.hashmap:
-                    # TODO: Comment out the below line after Debugging
                     assert own_name not in self.hashmap, ('Name Conflict in Hashmap. ' + \
                                                           own_name + 'already present\n' + foo)
                 self.hashmap [own_name] = widget (objectInfo.objType, name, parentName, accessible.getIndexInParent ())
@@ -439,13 +493,14 @@ class appmap:
                    childRole == Accessibility.ROLE_ALERT or childRole == Accessibility.ROLE_FONT_CHOOSER or \
                    childRole == Accessibility.ROLE_FILE_CHOOSER:
                 self.objectInst.reset ()
+            #time.sleep (1.0 / pow (10, 2))
             self.dumpTree (child, parentName)
             child.unref ()
 
     def isRoleWindowType (self, _role):
         if _role == Accessibility.ROLE_FRAME or _role == Accessibility.ROLE_DIALOG or \
                _role == Accessibility.ROLE_ALERT or _role == Accessibility.ROLE_FONT_CHOOSER or \
-               _role == Accessibility.ROLE_FILE_CHOOSER:
+               _role == Accessibility.ROLE_FILE_CHOOSER or _role == Accessibility.ROLE_WINDOW:
             return True
         else:
             False
@@ -473,10 +528,12 @@ class appmap:
                _role == Accessibility.ROLE_FILE_CHOOSER:
             _windowType = 'dlg'
         else:
+            if ldtpDebug:
+                print 'Not a window type'
             return None
         if _windowName and _windowName != '':
-            return _windowType + self.escapeChars (_windowName, False)
-        return _windowType + '0'
+            return _windowType + escapeChars (_windowName, False)
+        return _windowType + '0' # FIXME: Index has to be increased, instead of fixed 0
 
     def getParentNameAppmapFormat (self, accessible):
         # Provide the parent's accessible handle
@@ -484,7 +541,7 @@ class appmap:
         if accessible is None:
             return None
         _parentInfo = self.getObjectInfo (accessible, None)
-        _parentName = _parentInfo.prefix + self.escapeChars (accessible.name)
+        _parentName = _parentInfo.prefix + escapeChars (accessible.name)
         accessible.unref ()
         return _parentName
 
@@ -494,7 +551,7 @@ class appmap:
         _objInfo = self.getObjectInfo (accessible, \
                                        self.getParentNameAppmapFormat (accessible.parent))
         if accessible.name and accessible.name != '' and _objInfo:
-            _objName = _objInfo.prefix + self.escapeChars (accessible.name)
+            _objName = _objInfo.prefix + escapeChars (accessible.name)
             return _objName
         return None
 
@@ -505,7 +562,7 @@ class appmap:
                                        self.getParentNameAppmapFormat (accessible.parent))
         _relationName = self.getRelationName (accessible)
         if _relationName and _relationName != '' and _objInfo:
-            _objName = _objInfo.prefix + self.escapeChars (_relationName)
+            _objName = _objInfo.prefix + escapeChars (_relationName)
             return _objName
         return None
 
@@ -520,23 +577,27 @@ class appmap:
             _windowName = ''
             if self.isRoleWindowType (_role):
                 _windowName = _name = self.getWinNameAppmapFormat (accessible.name, _role)
+                self.hierarchy = '%(window)s#%(objtype)s:%(index)d:%(name)s;%(hierarchy)s' % \
+                                {'window' : _windowName, 'objtype' : objectInfo.objType,
+                                'index' : accessible.getIndexInParent (), \
+                                'name' : _name, 'hierarchy' : self.hierarchy}
+                return
+#            else:
+#                _name = self.getObjNameAppmapFormat (accessible)
+#                _windowHandle = self.getWindowHandle (accessible)
+#                if _windowHandle is not None:
+#                    _windowName = self.getWinNameAppmapFormat (_windowHandle.name, _windowHandle.getRole ())
+#                    _windowHandle.unref ()
+        if objectInfo.objType != 'unknown' and accessible.getIndexInParent () != -1:
+            if self.hierarchy == '':
+                self.hierarchy = '%(objtype)s:%(index)d' % \
+                                 {'objtype' : objectInfo.objType, 'index' : accessible.getIndexInParent ()}
             else:
-                _name = self.getObjNameAppmapFormat (accessible)
-                _windowHandle = self.getWindowHandle (accessible)
-                if _windowHandle is not None:
-                    _windowName = self.getWinNameAppmapFormat (_windowHandle.name, _windowHandle.getRole ())
-                    _windowHandle.unref ()
-            self.hierarchy = '%(window)s#%(objtype)s:%(index)d:%(name)s;%(hierarchy)s' % \
-                             {'window' : _windowName, 'objtype' : objectInfo.objType,
-                              'index' : accessible.getIndexInParent (), \
-                              'name' : _name, 'hierarchy' : self.hierarchy}
-            return
-        if self.hierarchy == '':
-            self.hierarchy = '%(objtype)s:%(index)d' % \
-                             {'objtype' : objectInfo.objType, 'index' : accessible.getIndexInParent ()}
+                self.hierarchy = '%(objtype)s:%(index)d;%(hierarchy)s' % \
+                                 {'objtype' : objectInfo.objType, 'index' : accessible.getIndexInParent (), 'hierarchy' : self.hierarchy}
         else:
-            self.hierarchy = '%(objtype)s:%(index)d;%(hierarchy)s' % \
-                             {'objtype' : objectInfo.objType, 'index' : accessible.getIndexInParent (), 'hierarchy' : self.hierarchy}
+            if ldtpDebug:
+                print 'Unknown objectInfo'
         _parent = accessible.parent
         if _parent is None:
             return
@@ -590,6 +651,32 @@ class appmap:
             child.unref ()
         return None
 
+    def getChildWindowHandle (self, accessible,  name):
+        if accessible is None:
+            return None
+        if ldtpDebug:
+            print 'childCount',  accessible.childCount
+        for i in range (0, accessible.childCount):
+            try:
+                _app = accessible.getChildAtIndex (i)
+            except:
+                if ldtpDebug:
+                    print traceback.print_exc ()
+                continue
+            if _app is None:
+                continue
+            _role = _app.getRole ()
+            if ldtpDebug:
+                print 'Role: ',  _app.getRoleName (),  name
+                if _app.name != None:
+                    print '_app.name',  _app.name,  self.getWinNameAppmapFormat (_app.name,  _role)
+            if _app.name is not None and _app.name != '' and \
+                (re.search (name,  _app.name) != None or \
+                re.search (name,  self.getWinNameAppmapFormat (_app.name,  _role)) != None):
+                return _app
+            _app.unref ()
+        return None
+
     def getWindowHandle (self, accessible):
         if accessible is None:
             return None
@@ -597,27 +684,63 @@ class appmap:
         if _role == Accessibility.ROLE_FILE_CHOOSER or _role == Accessibility.ROLE_FONT_CHOOSER or \
                _role == Accessibility.ROLE_DIALOG or _role == Accessibility.ROLE_FRAME or \
                _role == Accessibility.ROLE_ALERT or _role == Accessibility.ROLE_WINDOW:
-            return accessible
-        _parent = accessible.parent
-        _handle = self.getWindowHandle (_parent)
-        if _handle != _parent:
+                return accessible
+        _parent = None
+        _handle = None
+        try:
+            _parent = accessible.parent
+            _handle = self.getWindowHandle (_parent)
+        except:
+            pass
+        if _handle != _parent and _parent is not None:
             _parent.unref ()
         return _handle
 
-    def escapeChars (self, str2escape, escapeDot = True):
-        str2escape = re.sub ("(?i) +", "", str2escape)
-        str2escape = re.sub ("(?i)\n+", "", str2escape)
-        if escapeDot:
-            str2escape = re.sub ("(?i)\.+", "", str2escape)
-            str2escape = re.sub ("(?i)_+", "", str2escape)
-            str2escape = re.sub ("(?i):+", "", str2escape)
-        return str2escape
+    def getContextHandle ():
+        return None
 
-    def escapeChar (self, str2escape, char):
-        if char == '*':
-            char = ".*"
-        str2escape = re.sub ("(?i)" + char + "+", "", str2escape)
-        return str2escape
+    def getComponentHandle ():
+        return None
+
+    def getAppHandle (self,  appName):
+        # App handle should be freed, if not None
+        if appName == None or appName == '':
+            return None
+        _desktop = self.registry.getDesktop (0)
+        if _desktop is None:
+            return None
+        _app = ''
+        for i in range (0, _desktop.childCount):
+            try:
+                _app = _desktop.getChildAtIndex (i)
+            except:
+                if ldtpDebug:
+                    print traceback.print_exc ()
+                continue
+            if _app is None:
+                continue
+            _name = ''
+            if _app.name is not None and _app.name != "":
+                if ldtpDebug:
+                    print 'DEBUG _app.name, _app.getRoleName',  _app.name,  _app.getRoleName ()
+                _name = self.getWinNameAppmapFormat (_app.name,  _app.getRole ())
+                if ldtpDebug:
+                    print '_name',  _name
+            else:
+                _app.unref ()
+                continue
+            if ldtpDebug:
+                print 'appName',  appName,  _name,  _app.name
+            if _name != None and re.search (appName,  _name) != None or \
+                re.search (appName,  _app.name) != None:
+                return _app
+            else:
+                _appHandle = self.getChildWindowHandle (_app,  appName)
+                if _appHandle != None:
+                    _app.unref ()
+                    return _appHandle
+            _app.unref ()
+        return None
 
     def searchBasedOnKey (self, key, searchObjInfo):
         if key is None or searchObjInfo is None:
@@ -688,7 +811,7 @@ class appmap:
             return True
         if re.search (' ', data) is None:
             return False
-        tmp = self.escapeChar (data, ' ')
+        tmp = escapeChar (data, ' ')
         if key == tmp:
             return True
         if re.match ("(.*" + tmp + "){1}", key) or re.match (key, tmp):
