@@ -27,7 +27,13 @@
 #############################################################################
 
 import sys, commands, os, string, ldtp, time, xml.dom.minidom
-import threading, re
+import threading, re, tempfile
+
+statGrabMsg = None
+try:
+	import statgrab
+except ImportError:
+	statGrabMsg = 'pystatgrab package not installed'
 
 def tuplelist2list (lst):
         d = []
@@ -36,6 +42,14 @@ def tuplelist2list (lst):
                         d.append (lst[x-1][y-1])
         return d
 
+def getFullPath (path):
+    if path[0] == "~":
+        path = os.path.expanduser (path)
+    elif path[0] == ".":
+        path = os.path.abspath (path)
+
+    return path
+
 def imagecompare (imgfile1, imgfile2):
 	try:
 		import ImageChops, Image
@@ -85,13 +99,30 @@ def winname_to_winid (winname):
 	winid = string.split (winid, ')')[0]
 	return winid
 
-def imagecapture (winname, outfile, resolution1 = '1024', resolution2 = '768', x = '0', y = '0'):
-	winid = winname_to_winid (winname)
-	winid = '0x' + winid
-	cmd = 'import -window ' + winid + ' -crop ' + resolution1 + 'x' + resolution2 + '+' + x+ '+' + y  + ' ' + outfile
+def imagecapture (winName = None, outFile = None, resolution1 = '1024', resolution2 = '768', x = '0', y = '0'):
+	# winname == None, let us capture the entire screen
+	# if output file name is not passed, then a random file name will be generated in
+	# /tmp and it will be returned
+	if winName == None:
+		winid = 'root'
+	else:
+		status = commands.getstatusoutput ('which digwin')
+		if status [0] != 0:
+			raise ldtp.LdtpExecutionError ('To capture invidual window, you need to install digwin of LTFX')
+		winid = winname_to_winid (winname)
+		winid = '0x' + winid
+	if outFile == None:
+		file = tempfile.NamedTemporaryFile ()
+		tmpFile = file.name + '.png'
+		file.close ()
+	else:
+		tmpFile = outFile
+	cmd = 'import -window ' + winid + ' -crop ' + resolution1 + 'x' + resolution2 + '+' + x + '+' + y  + ' ' + tmpFile
 	status = commands.getstatusoutput (cmd)
-	if status[0] != 0:
+	if status [0] != 0:
 		ldtp.log ('Unable to capture', 'error')
+		return None
+	return tmpFile
 
 def blackoutregion (infile, outfile, topx, topy, botx, boty):
 	try:
@@ -157,22 +188,19 @@ class LdtpDataFileParser:
 				self.taglist.append (self.getText (data.childNodes))
 		return self.taglist
 
-# Capturing Memory and CPU Utilization statistics for an application and its related processes
-# EXAMPLE USAGE:
-# 	xstats = pstats('evolution', 2)
-# 	Start Logging by calling start
-# 	xstats.start()
-# 	Stop the process statistics gathering thread by calling the stopstats method
-# 	xstats.stop()
-
 class pstats (threading.Thread):
+	"""Capturing Memory and CPU Utilization statistics for an application and its related processes
+	EXAMPLE USAGE:
+		xstats = pstats('evolution', 2)
+		# Start Logging by calling start
+		xstats.start()
+		# Stop the process statistics gathering thread by calling the stopstats method
+		xstats.stop()"""
+
 	def __init__ (self, appname, inter = 2):
-		try:
-			import statgrab
-		except ImportError:
-			msg = 'libstatgrab package not installed'
-			ldtp.log (msg, 'error')
-			raise ldtp.LdtpExecutionError (msg)
+		if statGrabMsg is not None:
+			ldtp.log (statGrabMsg, 'error')
+			raise ldtp.LdtpExecutionError (statGrabMsg)
 		threading.Thread.__init__ (self)
 		self.processname = appname
 		self.interval = inter
@@ -270,13 +298,3 @@ def typekey (window_name):
 	#Type the given text in the focused window
 	cmd = "ltfx -e 'type \"" +  window_name + "\"\'"	
 	return execute (cmd)
-
-# General functions
-def wait (seconds = 5):
-	try:
-		time.sleep (seconds)
-	except TypeError:
-		time.sleep (5)
-
-def waitnanoseconds (nanoSeconds = 30000.0):
-	time.sleep (nanoSeconds / pow (10, 9))
