@@ -31,6 +31,8 @@
 #include "ldtp-command.h"
 #include "ldtp-gui-comp.h"
 
+extern gboolean ldtp_debug;
+
 static LDTPErrorCode
 select_event_index (Accessible *object, long event_index, FILE *log_fp)
 {
@@ -92,9 +94,10 @@ select_event (Accessible *object, char *event_name, FILE *log_fp)
 		if (get_object_type (event) == CALENDAR_EVENT) {
 			char *name;
 			name = Accessible_getName (event);
-			if (name)
-				g_print ("Name: %s - Event Name: %s\n", name, event_name);
-			if (event_name && (g_utf8_collate (name,app_name) > 0) && Accessible_isComponent (event)) {
+			if (ldtp_debug && name && event_name && app_name)
+				g_print ("Name: %s - Event Name: %s - app_name - %s\n", name, event_name, app_name);
+
+			if (event_name && (g_utf8_collate (name, app_name) > 0) && Accessible_isComponent (event)) {
 				SPIBoolean flag = FALSE;
 				AccessibleComponent *accessible_component;
 
