@@ -42,6 +42,8 @@ extern gint ldtp_gui_timeout;
 extern gint ldtp_obj_timeout;
 extern GHashTable *event_notifier;
 extern GHashTable *client_context;
+extern gboolean ldtp_script_service;
+extern gboolean ldtp_external_xml_file;
 
 extern pthread_mutex_t cb_mutex;
 
@@ -90,6 +92,26 @@ unregister_window_creation_event (LDTPClientContext *cctxt, LDTPErrorCode *err)
 	*err = LDTP_ERROR_SUCCESS;
 }
 
+static char*
+write2file (char *content, long len)
+{
+	char filename [] = "/tmp/LDTP-XML-XXXXXX";
+	int fd;
+
+	if (content == NULL || len <= 0) {
+		g_print ("Argument NULL\n");
+		return NULL;
+	}
+
+	if ((fd = mkstemp (filename)) < 0) {
+		g_print ("Unable to create tmp file - mkstemp\n");
+		return NULL;
+	}
+	write (fd, content, len);
+	close (fd);
+	return g_strdup (filename);
+}
+
 void
 generate_response_packet (LDTPClientContext* cctxt, LDTPErrorCode* err, char **resp_pckt, uint32_t *resp_size)
 {
@@ -140,18 +162,36 @@ generate_response_packet (LDTPClientContext* cctxt, LDTPErrorCode* err, char **r
 			DATA_END_ELEMENT_LEN +
 			cctxt->resp->data_len;
 		memset (buf1, 0x00, 16);
-		sprintf (buf1, "%o", cctxt->resp->data_len);
+		sprintf (buf1, "%ld", cctxt->resp->data_len);
 		resp_len += strlen (buf1);
 	}
 
-	if (cctxt->resp->data && cctxt->resp->data_len > 0) 
-		data_pckt = g_strconcat (DATA_ELEMENT, 
-					 ATTRIBUTE_LENGTH_ELEMENT,
-					 buf1, ATTRIBUTE_LENGTH_END_ELEMENT,
-					 ATTRIBUTE_VALUE_ELEMENT, 
-					 cctxt->resp->data,
-					 ATTRIBUTE_VALUE_END_ELEMENT, 
-					 DATA_END_ELEMENT, NULL);
+	if (cctxt->resp->data && cctxt->resp->data_len > 0) {
+		char *external_tmp_xml_file = getenv ("LDTP_EXTERNAL_TEMP_FILE");
+		if (!ldtp_script_service &&
+		    (ldtp_external_xml_file || (external_tmp_xml_file != NULL &&
+						g_ascii_strcasecmp (external_tmp_xml_file, "2") == 0)) &&
+		    cctxt->resp->data_len > 512) {
+			char *filename = write2file (cctxt->resp->data, cctxt->resp->data_len);
+			if (filename) {
+				data_pckt = g_strconcat (ATTRIBUTE_FILE_ELEMENT,
+							 ATTRIBUTE_NAME_ELEMENT,
+							 filename,
+							 ATTRIBUTE_NAME_END_ELEMENT,
+							 ATTRIBUTE_FILE_END_ELEMENT,
+							 NULL);
+				g_free (filename);
+			}
+		} else {
+			data_pckt = g_strconcat (DATA_ELEMENT,
+						 ATTRIBUTE_LENGTH_ELEMENT,
+						 buf1, ATTRIBUTE_LENGTH_END_ELEMENT,
+						 ATTRIBUTE_VALUE_ELEMENT, 
+						 cctxt->resp->data,
+						 ATTRIBUTE_VALUE_END_ELEMENT, 
+						 DATA_END_ELEMENT, NULL);
+		}
+	}
 
 	resp_xml = g_strdup_printf ("%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
 				    XML_HEADER, RESPONSE_ELEMENT,
@@ -167,7 +207,8 @@ generate_response_packet (LDTPClientContext* cctxt, LDTPErrorCode* err, char **r
 				    data_pckt ? data_pckt : "",
 				    RESPONSE_END_ELEMENT);
 
-	g_free (data_pckt);
+	if (data_pckt)
+		g_free (data_pckt);
 
 	*err = LDTP_ERROR_SUCCESS;
 
@@ -220,7 +261,7 @@ generate_notification_packet (LDTPClientContext* cctxt, LDTPErrorCode* err, char
 			DATA_END_ELEMENT_LEN +
 			cctxt->resp->data_len;
 		memset (buf1, 0x00, 16);
-		sprintf (buf1, "%o", cctxt->resp->data_len);
+		sprintf (buf1, "%ld", cctxt->resp->data_len);
 		resp_len += strlen (buf1);
 	}
 
@@ -313,6 +354,67 @@ add_item_to_list (gpointer key, gpointer value, gpointer list)
 	*user_data = data;
 }
 
+static gboolean
+is_role_matching (gpointer key, gpointer value, gpointer list)
+{
+        if (!value || !list)
+                return FALSE;
+	if (g_utf8_collate (value, list) == 0)
+		return TRUE;
+	return FALSE;
+}
+
+static void
+add_matching_item_to_list (gpointer key, gpointer value, gpointer list)
+{
+	MatchingList *list_item = list;
+	gchar *tmp = NULL;
+	gchar *data = NULL;
+	gchar *label = NULL;
+	gchar *escaped_xml_data = NULL;
+        if (!key || !value || !list)
+                return;
+	data = list_item->data;
+	if (ldtp_debug)
+		g_print ("key: %s\n", (gchar *) key);
+	/*
+	  list_item->role == NULL, let us add all of the entries to the list
+	*/
+	if (list_item->role == NULL || g_hash_table_find (value, is_role_matching, list_item->role)) {
+		label = get_property (value, "label_by", NULL);
+		if (!label)
+			label = get_property (value, "label", NULL);
+		if (!label)
+			escaped_xml_data = escape_xml_character (key);
+		else {
+			if (g_utf8_strchr (label, -1, '_')) {
+				tmp = escape_character (label, '_');
+				escaped_xml_data = escape_xml_character (tmp);
+				g_free (tmp);
+			} else
+				escaped_xml_data = escape_xml_character (label);
+		}
+		if (list_item->name && g_utf8_collate (label, list_item->name) != 0)
+			/*
+			  If name is not NULL and its label doesn't match, name, then
+			  let us not include in the list
+			  For example:
+			  name = Find, role = push_button in gedit application, just get,
+			  the label is Find and role type is push_button.
+			*/
+			return;
+		if (data) {
+			tmp = g_strdup_printf ("%s<OBJECT>%s</OBJECT>", data, escaped_xml_data);
+			g_free (data);
+			data = tmp;
+		}
+		else
+			data = g_strdup_printf ("<OBJECT>%s</OBJECT>", escaped_xml_data);
+		g_free (escaped_xml_data);
+		list_item->data = data;
+	}
+}
+
 #ifdef ENABLE_LOCALIZATION
 static void
 createpo (gchar *package_name, gchar *locale_dir, gchar *file_name,
@@ -604,7 +706,7 @@ handle_request (LDTPClientContext* cctxt,
 		*err = LDTP_ERROR_INVALID_COMMAND;
 		return;
 	}
-	ldtp_nsleep (0, 30000);
+	sleep (1);
 	switch (ldtp_req->command) {
 	case LDTP_CMD_SETLOCALE:
 		if (cctxt->locale_lang)
@@ -824,7 +926,17 @@ handle_request (LDTPClientContext* cctxt,
 	case LDTP_CMD_VERIFYMENUCHECK:
 	case LDTP_CMD_VERIFYMENUUNCHECK:
 	case LDTP_CMD_SELECTMENUITEM:
-		if (g_utf8_strchr ((gchar *)ldtp_req->component, -1, ';')) {
+		if (g_utf8_strchr (ldtp_req->component, -1, ';')) {
+			gchar **token = NULL;
+			token = g_strsplit (ldtp_req->component, ";", 2);
+			gchar *tmp = token [1];
+			cctxt->req->arg_list = g_slist_append (cctxt->req->arg_list, 
+							       g_strdup (tmp));
+			g_free (ldtp_req->component);
+			ldtp_req->component = g_strdup (token [0]);
+			g_strfreev (token);
+
+			/*
 			char *token = NULL;
 			char *rest_token = NULL;
 			char *rest_params = NULL;
@@ -849,6 +961,7 @@ handle_request (LDTPClientContext* cctxt,
 			}
 			cctxt->req->arg_list = g_slist_append (cctxt->req->arg_list, 
 							       rest_params);
+			*/
 		}
 		break;
 
@@ -1023,7 +1136,79 @@ handle_request (LDTPClientContext* cctxt,
 		}
 		return;
 	}
+	case LDTP_CMD_GETCHILD: {
+		char *data = NULL;
+		char *name = cctxt->req->component;
+		char *role = NULL;
+		MatchingList list;
+		GHashTable *context_ht = NULL;
 
+		if (!cctxt->req->context) {
+			*err = LDTP_ERROR_ARGUMENT_NULL;
+			g_print ("%s %s %d\n", ldtp_error_get_message (*err), __FILE__, __LINE__);
+			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
+			return;
+		}
+		if (!cctxt->app_map) {
+                        /* if appmap not already initialised, do it now */
+                        accessible = ldtp_gui_get_gui_handle (cctxt, err);
+                        ldtp_gui_free_gui_handle (accessible);
+                        if (*err != LDTP_ERROR_SUCCESS) {
+				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
+                                return;
+                        }
+		}
+		context_ht = get_object_def (cctxt->app_map, (char *)cctxt->req->context, cctxt->log_fp, TRUE);
+		if (!context_ht) {
+			update_cur_window_appmap_handle (cctxt,
+							 err);
+			context_ht = get_object_def (cctxt->app_map, (char *)cctxt->req->context, cctxt->log_fp, TRUE);
+			if (*err != LDTP_ERROR_SUCCESS) {
+				g_print ("Unable to update context: %s in appmap", (char *)cctxt->req->context);
+				return;
+			}
+		}
+		if (!context_ht) {
+			g_print ("Unable to find context\n");
+			*err = LDTP_ERROR_UNABLE_TO_GET_CONTEXT_HANDLE;
+			g_print ("%s\n", ldtp_error_get_message (*err));
+			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
+			return;
+		}
+		role = g_slist_nth_data (cctxt->req->arg_list, 0);
+		list.data = data;
+		list.name = name == NULL ? NULL : g_utf8_collate (name,
+								  cctxt->req->context) == 0 ? NULL : name;
+		list.role = role;
+		g_hash_table_foreach (context_ht, &add_matching_item_to_list, (gpointer) &list);
+		cctxt->resp->data = list.data;
+		if (cctxt->resp->data) {
+			char *tmp = NULL;
+			/*
+			  FRAME the output in XML format
+			*/
+			tmp = g_strdup_printf ("%s<OBJECTLIST>%s</OBJECTLIST>", XML_HEADER, cctxt->resp->data);
+			if (tmp) {
+				g_free (cctxt->resp->data);
+				cctxt->resp->data = tmp;
+				if (ldtp_debug)
+					g_print ("LIST: %s\n", tmp);
+				cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
+			}
+			else {
+				g_free (cctxt->resp->data);
+				*err = LDTP_ERROR_UNABLE_TO_GET_CHILD_WITH_PROVIDED_ROLE;
+				g_print ("%s\n", ldtp_error_get_message (*err));
+				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
+			}
+		}
+		else {
+			*err = LDTP_ERROR_UNABLE_TO_GET_CHILD_WITH_PROVIDED_ROLE;
+			g_print ("%s\n", ldtp_error_get_message (*err));
+			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
+		}
+		return;
+	}
 	case LDTP_CMD_GETOBJECTINFO: {
 		char *data = NULL;
 		GHashTable *context_ht = NULL;
@@ -1308,6 +1493,10 @@ handle_request (LDTPClientContext* cctxt,
 		*err = scroll_bar_main (cctxt, cctxt->req->command);
 		break;
 
+	case SPI_ROLE_SCROLL_PANE:
+		*err = scroll_pane_main (cctxt, cctxt->req->command);
+		break;
+
 	case SPI_ROLE_SLIDER:
 		*err = slider_main (cctxt, cctxt->req->command);
 		break;
@@ -1339,6 +1528,7 @@ handle_request (LDTPClientContext* cctxt,
 			break;
 		}
 		*err = LDTP_ERROR_ROLE_NOT_IMPLEMENTED;
+		g_print ("Role not implemented: %s %d", __FILE__, __LINE__);
 		break;
 	}
 
@@ -1418,7 +1608,7 @@ handle_client (void *ptr)
 			}
 
 			if (packet)
-				g_print ("Data read %d, packet-len = %d, bytes read = %d, data: %s\n", 
+				g_print ("Data read %d, packet-len = %d, bytes read = %ld, data: %s\n", 
 					 pckt_len, packet_len, bytes_read, packet);
 			packet += bytes_read;
 			packet_read += bytes_read;
