@@ -36,6 +36,7 @@
 
 typedef struct _LDTPClientContext LDTPClientContext;
 typedef struct _Packet Packet;
+typedef struct _MatchingList MatchingList;
 
 typedef struct _LDTPGuiHandle LDTPGuiHandle;
 struct _LDTPGuiHandle {
@@ -62,6 +63,12 @@ struct _Packet {
 	uint32_t len;
 };
 
+struct _MatchingList {
+	gchar *data;
+	gchar *name;
+	gchar *role;
+};
+
 void *handle_client (void *ptr);
 
 void generate_response_packet (LDTPClientContext* cctxt, LDTPErrorCode* err,
