@@ -177,7 +177,8 @@ enum _LDTPCommandId {
 	LDTP_CMD_PRESS,
 	LDTP_CMD_REMOVECALLBACK,
 	LDTP_CMD_GUITIMEOUT,
-	LDTP_CMD_OBJTIMEOUT
+	LDTP_CMD_OBJTIMEOUT,
+	LDTP_CMD_GETCHILD
 };
 
 #endif
