@@ -562,6 +562,9 @@ ldtp_error_get_message (LDTPErrorCode error)
 	case LDTP_ERROR_SET_OBJ_TIMEOUT_FAILED:
 		msg = "Unable to set obj timeout period";
 		break;
+	case LDTP_ERROR_UNABLE_TO_GET_CHILD_WITH_PROVIDED_ROLE:
+		msg = "Unable to get child with provided role";
+		break;
 	default:
 		msg = "Error code not found";
 		break;
