@@ -202,7 +202,8 @@ typedef enum _LDTPErrorCode {
 	LDTP_ERROR_CLIENT_DISCONNECTED,
 	LDTP_ERROR_EVENT_NOTIFIER_NOT_ENABLED,
 	LDTP_ERROR_SET_GUI_TIMEOUT_FAILED,
-	LDTP_ERROR_SET_OBJ_TIMEOUT_FAILED
+	LDTP_ERROR_SET_OBJ_TIMEOUT_FAILED,
+	LDTP_ERROR_UNABLE_TO_GET_CHILD_WITH_PROVIDED_ROLE
 } LDTPErrorCode;
 
 const char* ldtp_error_get_message (LDTPErrorCode error);
