@@ -338,7 +338,7 @@ wait_till_object_state_contains (Accessible *object, int control_type, FILE *log
 	}
 
 	while ((difftime (cur, start) < time_wait) && (state != NULL)) {
-		ldtp_nsleep (0, 30000);
+		sleep (1);
 
 		cur = time (NULL);
 		flag = TRUE;
@@ -784,18 +784,26 @@ get_relation_name (Accessible *object, long *len)
 			    relation_type == SPI_RELATION_LABEL_FOR) {
 				char *name = NULL;
 				Accessible *tmp_obj;
-				tmp_obj = AccessibleRelation_getTarget (relation[i], i);
-				name = Accessible_getName (tmp_obj);
+				//tmp_obj = AccessibleRelation_getTarget (relation[i], i);
+				tmp_obj = AccessibleRelation_getTarget (relation[i], j);
+				if (tmp_obj)
+					name = Accessible_getName (tmp_obj);
 				if (name == NULL) {
-					Accessible_unref (tmp_obj);
+					if (ldtp_debug)
+						g_print ("Relation name: NULL - %ld - %ld\n", i, j);
+					if (tmp_obj)
+						Accessible_unref (tmp_obj);
 					for (k = 0; relation[k] != NULL; k++)
 						AccessibleRelation_unref (relation[k]);
 					g_free (relation);
 					return NULL;
 				}
 				if (g_ascii_strcasecmp (name, "") == 0) {
+					if (ldtp_debug)
+						g_print ("Relation name: EMPTY string\n");
 					SPI_freeString (name);
-					Accessible_unref (tmp_obj);
+					if (tmp_obj)
+						Accessible_unref (tmp_obj);
 					for (k = 0; relation[k] != NULL; k++)
 						AccessibleRelation_unref (relation[k]);
 					g_free (relation);
@@ -1264,6 +1272,124 @@ update_cur_context_appmap_handle (LDTPClientContext *cctxt,
 	*err = LDTP_ERROR_SUCCESS;
 }
 
+static Accessible*
+ldtp_get_component_handle (LDTPClientContext *cctxt, char *window_prop, gchar *context, char *component, LDTPErrorCode* err)
+{
+	Accessible *context_handle = NULL;
+	Accessible *component_handle = NULL;
+
+	char *class_type  = NULL;
+	struct node *head = NULL;
+
+	GHashTable *cur_window;
+	GHashTable *cur_component;
+
+	int time_wait = ldtp_obj_timeout;
+	time_t start, cur;
+	gboolean flag = FALSE;
+
+	start = cur = time (NULL);
+
+	if (!ldtp_obj_timeout) {
+		time_wait = 5; // Max wait for 5 seconds
+	}
+
+	while (difftime (cur, start) < time_wait) {
+		/*
+		  To avoid runtime remap function to be called, let us call remap here
+		*/
+		g_print ("Updating appmap - component\n");
+		if (cctxt->window_name) {
+			g_free (cctxt->window_name);
+			cctxt->window_name = NULL;
+		}
+		update_cur_window_appmap_handle (cctxt, err);
+		//updated_appmap = TRUE;
+		component_handle = NULL;
+		context_handle = get_accessible_context_handle (cctxt->app_handle, window_prop);
+		if (!context_handle) {
+			context_handle = get_accessible_context_handle (cctxt->app_handle, context);
+		}
+		if (context_handle) {
+			char *name = Accessible_getName (context_handle);
+			if (name) {
+				char *tmp_context = get_window_name_in_appmap_format (name, Accessible_getRole (context_handle));
+				if (tmp_context) {
+					cur_window = get_object_def (cctxt->app_map, tmp_context, cctxt->log_fp, TRUE);
+					cur_component = get_object_def (cur_window, component, cctxt->log_fp, FALSE);
+					head = trace_path_to_parent (cur_window, tmp_context, cur_component,
+								     &class_type, cctxt->log_fp, cctxt, err);
+					component_handle = get_accessible_component_handle (head, context_handle, cctxt->log_fp);
+					g_free (tmp_context);
+				}
+				SPI_freeString (name);
+			}
+			Accessible_unref (context_handle);
+			g_free (head);
+		}
+		if (component_handle) {
+			char *role = Accessible_getRoleName (component_handle);
+			if (role) {
+				char *str1 = NULL;
+				char *str2 = NULL;
+				if (g_utf8_strchr (role, -1, ' ') != NULL) {
+					str1 = escape_character (role, ' ');
+				} else {
+					str1 = g_strdup (role);
+				}
+				if (g_utf8_strchr (class_type, -1, '_') != NULL) {
+					str2 = escape_character (class_type, '_');
+				} else {
+					str2 = g_strdup (class_type);
+				}
+				if (str1 && str2) {
+					if (ldtp_debug)
+						g_print ("str1: %s - str2: %s\n", str1, str2);
+					if (g_utf8_collate (str1, str2) != 0) {
+						char *tmp_str1 = g_utf8_casefold (str1, -1);
+						char *tmp_str2 = g_utf8_casefold (str2, -1);
+						if (tmp_str1 && tmp_str2) {
+							if (ldtp_debug)
+								g_print ("tmp_str1: %s - tmp_str2: %s\n", tmp_str1, tmp_str2);
+							if (g_utf8_collate (tmp_str1, tmp_str2) != 0) {
+								// Role types are not same, so we need to rescan
+								Accessible_unref (component_handle);
+								component_handle = NULL;
+							}
+						} else {
+							// Role types are not same, so we need to rescan
+							Accessible_unref (component_handle);
+							component_handle = NULL;
+						}
+						if (tmp_str1)
+							g_free (tmp_str1);
+						if (tmp_str2)
+							g_free (tmp_str2);
+					}
+				}
+				if (str1)
+					g_free (str1);
+				if (str2)
+					g_free (str2);
+				SPI_freeString (role);
+			}
+			class_type = NULL;
+		}
+		if (!component_handle || Accessible_getRole (component_handle) == SPI_ROLE_UNKNOWN) {
+			if (component_handle) {
+				Accessible_unref (component_handle);
+				component_handle = NULL;
+			}
+			sleep (1);
+			cur = time (NULL);
+		} else {
+			flag = TRUE;
+			break;
+		}
+	}
+	return component_handle;
+}
+
 /*
   Get gui handle based on context and compoment
 */
@@ -1277,7 +1403,7 @@ ldtp_gui_get_gui_handle (LDTPClientContext* cctxt,
 	char *component   = NULL;
 	char *class_type  = NULL;
 	char *window_prop = NULL;
-	struct node *head;
+	struct node *head = NULL;
 	gboolean flag = FALSE;
 	gboolean updated_appmap = FALSE;
 	AccessibleRole class;
@@ -1431,7 +1557,7 @@ ldtp_gui_get_gui_handle (LDTPClientContext* cctxt,
 				cur_component = get_object_def (cur_window, component, cctxt->log_fp, FALSE);
 
 				if (!cur_component) {
-					ldtp_nsleep (0, 30000);
+					sleep (1);
 
 					cur = time (NULL);
 				} else {
@@ -1529,9 +1655,25 @@ ldtp_gui_get_gui_handle (LDTPClientContext* cctxt,
 				if (ldtp_debug)
 					g_print ("str1: %s - str2: %s\n", str1, str2);
 				if (g_utf8_collate (str1, str2) != 0) {
-					// Role types are not same, so we need to rescan
-					Accessible_unref (component_handle);
-					component_handle = NULL;
+					char *tmp_str1 = g_utf8_casefold (str1, -1);
+					char *tmp_str2 = g_utf8_casefold (str2, -1);
+					if (tmp_str1 && tmp_str2) {
+						if (ldtp_debug)
+							g_print ("tmp_str1: %s - tmp_str2: %s\n", tmp_str1, tmp_str2);
+						if (g_utf8_collate (tmp_str1, tmp_str2) != 0) {
+							// Role types are not same, so we need to rescan
+							Accessible_unref (component_handle);
+							component_handle = NULL;
+						}
+					} else {
+						// Role types are not same, so we need to rescan
+						Accessible_unref (component_handle);
+						component_handle = NULL;
+					}
+					if (tmp_str1)
+						g_free (tmp_str1);
+					if (tmp_str2)
+						g_free (tmp_str2);
 				}
 			}
 			if (str1)
@@ -1544,93 +1686,11 @@ ldtp_gui_get_gui_handle (LDTPClientContext* cctxt,
 	}
 
 	if (!component_handle || Accessible_getRole (component_handle) == SPI_ROLE_UNKNOWN) {
-		int time_wait = ldtp_obj_timeout;
-		time_t start, cur;
-		gboolean flag = FALSE;
-
-		if (component_handle) {
+		if (component_handle)
 			Accessible_unref (component_handle);
-			component_handle = NULL;
-		}
-		/*
-		  Copied from LTFX source code
-		*/
-		start = cur = time (NULL);
-
-		if (!ldtp_obj_timeout) {
-			time_wait = 5; // Max wait for 5 seconds
-		}
-
-		while (difftime (cur, start) < time_wait) {
-			/*
-			  To avoid runtime remap function to be called, let us call remap here
-			*/
-			g_print ("Updating appmap - component\n");
-			if (cctxt->window_name) {
-				g_free (cctxt->window_name);
-				cctxt->window_name = NULL;
-			}
-			update_cur_window_appmap_handle (cctxt, err);
-			updated_appmap = TRUE;
-			context_handle = get_accessible_context_handle (cctxt->app_handle, window_prop);
-			if (!context_handle) {
-				context_handle = get_accessible_context_handle (cctxt->app_handle, context);
-			}
-			if (context_handle) {
-				cur_window = get_object_def (cctxt->app_map, context, cctxt->log_fp, TRUE);
-				cur_component = get_object_def (cur_window, component, cctxt->log_fp, FALSE);
-				head = trace_path_to_parent (cur_window, context, cur_component,
-							     &class_type, cctxt->log_fp, cctxt, err);
-				component_handle = get_accessible_component_handle (head, context_handle, cctxt->log_fp);
-				Accessible_unref (context_handle);
-				g_free (head);
-			}
-			if (component_handle) {
-				char *role = Accessible_getRoleName (component_handle);
-				if (role) {
-					char *str1 = NULL;
-					char *str2 = NULL;
-					if (g_utf8_strchr (role, -1, ' ') != NULL) {
-						str1 = escape_character (role, ' ');
-					} else {
-						str1 = g_strdup (role);
-					}
-					if (g_utf8_strchr (class_type, -1, '_') != NULL) {
-						str2 = escape_character (class_type, '_');
-					} else {
-						str2 = g_strdup (class_type);
-					}
-					if (str1 && str2) {
-						if (ldtp_debug)
-							g_print ("str1: %s - str2: %s\n", str1, str2);
-						if (g_utf8_collate (str1, str2) != 0) {
-							// Role types are not same, so we need to rescan
-							Accessible_unref (component_handle);
-							component_handle = NULL;
-						}
-					}
-					if (str1)
-						g_free (str1);
-					if (str2)
-						g_free (str2);
-					SPI_freeString (role);
-				}
-				class_type = NULL;
-			}
-			if (!component_handle || Accessible_getRole (component_handle) == SPI_ROLE_UNKNOWN) {
-				if (component_handle) {
-					Accessible_unref (component_handle);
-					component_handle = NULL;
-				}
-				ldtp_nsleep (0, 30000);
-
-				cur = time (NULL);
-			} else {
-				flag = TRUE;
-				break;
-			}
-		}
+		component_handle = ldtp_get_component_handle (cctxt, window_prop, context, component, err);
 	}
+
 	if (!component_handle) {
 		if (ldtp_debug && component) {
 			g_sprintf (msg, "Unable to get object handle %s", component);
@@ -1649,11 +1709,48 @@ ldtp_gui_get_gui_handle (LDTPClientContext* cctxt,
 			SPI_freeString (name);
 		}
 		Accessible_unref (component_handle);
-		*err = LDTP_ERROR_OBJ_INFO_MISMATCH;
-		goto error;
+		g_print ("Let us try rescan\n");
+		context_handle = get_accessible_context_handle (cctxt->app_handle, window_prop);
+		if (!context_handle) {
+			context_handle = get_accessible_context_handle (cctxt->app_handle, context);
+		}
+		name = Accessible_getName (context_handle);
+		if (name) {
+			char *tmp_context = get_window_name_in_appmap_format (name, Accessible_getRole (context_handle));
+			SPI_freeString (name);
+			if (tmp_context) {
+				component_handle = ldtp_get_component_handle (cctxt, window_prop, tmp_context, component, err);
+				if (!component_handle) {
+					g_sprintf (msg, "Unable to get object handle %s", tmp_context);
+					g_print ("%s\n", msg);
+					*err = LDTP_ERROR_GET_OBJ_HANDLE_FAILED;
+					Accessible_unref (context_handle);
+					goto error;
+				}
+			}
+			cur_window = get_object_def (cctxt->app_map, tmp_context, cctxt->log_fp, TRUE);
+			cur_component = get_object_def (cur_window, component, cctxt->log_fp, FALSE);
+			g_free (tmp_context);
+			if (!is_object_matching (component_handle, cur_component, cctxt->log_fp)) {
+				name = Accessible_getName (component_handle);
+				if (name) {
+					g_sprintf (msg, "Object information: %s does not match with appmap entry", name);
+					g_print ("%s\n", msg);
+					SPI_freeString (name);
+				}
+				if (component_handle)
+					Accessible_unref (component_handle);
+				*err = LDTP_ERROR_OBJ_INFO_MISMATCH;
+				Accessible_unref (context_handle);
+				goto error;
+			}
+		}
+		Accessible_unref (context_handle);
 	}
 	ctxt_handle->class_id = 0;
-	class = Accessible_getRole (component_handle);
+	class = 0;
+	if (component_handle)
+		class = Accessible_getRole (component_handle);
 
 	if (class == SPI_ROLE_EXTENDED) {
 		char *name;
@@ -1881,8 +1978,7 @@ ldtp_gui_wait_till_gui_exist (LDTPClientContext* cctxt, LDTPErrorCode* err)
 	}
 
 	while ((difftime (cur, start) < time_wait) && (accessible == NULL)) {
-		ldtp_nsleep (0, 30000);
-
+		sleep (1);
 		cur = time (NULL);
 		accessible = ldtp_gui_get_gui_handle (cctxt, err);
 	}
@@ -1921,8 +2017,7 @@ ldtp_gui_wait_till_gui_not_exist (LDTPClientContext* cctxt, LDTPErrorCode* err)
 	accessible = ldtp_gui_get_gui_handle (cctxt, err);
 
 	while ((difftime (cur, start) < time_wait) && (accessible != NULL)) {
-		ldtp_nsleep (0, 30000);
-
+		sleep (1);
 		cur = time (NULL);
 		if (accessible) {
 			Accessible_unref (accessible->handle);
