@@ -33,6 +33,10 @@
 #include "ldtp-error.h"
 #include "ldtp-logger.h"
 
+#ifndef ENABLE_GOPTIONPARSE
+#include <getopt.h>
+#endif
+
 gint ldtp_script_port = 0;
 gint ldtp_gui_timeout = 0;
 gint ldtp_obj_timeout = 0;
@@ -40,6 +44,7 @@ gboolean ldtp_debug   = FALSE;
 static gboolean ldtp_usage   = FALSE;
 static gboolean ldtp_version = FALSE;
 gboolean ldtp_script_service = FALSE;
+gboolean ldtp_external_xml_file = TRUE;
 GHashTable *event_notifier = NULL;
 GHashTable *client_context = NULL;
 GHashTable *client_thread_pool = NULL;
@@ -47,9 +52,11 @@ static AccessibleEventListener *window_listener;
 pthread_mutex_t cb_mutex = PTHREAD_MUTEX_INITIALIZER;
 pthread_mutex_t client_thread_pool_mutex = PTHREAD_MUTEX_INITIALIZER;
 
-static GOptionEntry entries[] = 
+#ifdef ENABLE_GOPTIONPARSE
+static GOptionEntry entries [] = 
 	{
 		{ "gui-timeout", 'g', 0, G_OPTION_ARG_INT, &ldtp_gui_timeout, "Wait gui to appear / disappear till N seconds", "N" },
+		{ "no-external-xml-file", 'n', 0, G_OPTION_ARG_NONE, &ldtp_external_xml_file, "Don't use external temp XML file", NULL },
 		{ "obj-timeout", 'o', 0, G_OPTION_ARG_INT, &ldtp_obj_timeout, "Wait object to appear till N seconds", "N" },
 		{ "port", 'p', 0, G_OPTION_ARG_INT, &ldtp_script_port, "Start LDTP scripting engine on TCP port", "N" },
 		{ "script-engine", 's', 0, G_OPTION_ARG_NONE, &ldtp_script_service, "Start LDTP script execution engine as TCP service", NULL },
@@ -58,6 +65,7 @@ static GOptionEntry entries[] =
 		{ "version", 'V', 0, G_OPTION_ARG_NONE, &ldtp_version, "LDTP engine version", NULL },
 		{ NULL }
 	};
+#endif
 
 static LDTPClientContext*
 is_window_reg_for_events (char *context)
@@ -191,7 +199,8 @@ report_window_event  (const AccessibleEvent *event, void *user_data)
 static void
 signal_all_threads (gpointer key, gpointer value, gpointer userdata)
 {
-	pthread_kill ((pthread_t) value, SIGKILL);
+	if (value)
+		pthread_kill ((pthread_t) value, SIGKILL);
 }
 
 void
@@ -213,7 +222,7 @@ cleanup (int mysig)
 	*/
 	close_connection (get_server_socket (LDTP_SCRIPT_SERVER));
 
-	if (ldtp_script_service || ldtp_script_port)
+	if (ldtp_script_service)
 		goto quit;
 
 	tmpfile = get_tmp_file (LDTP_SCRIPT_SERVER);
@@ -302,9 +311,12 @@ close_connection (int fd)
 		char tmpfd [15];
 		pthread_mutex_lock (&client_thread_pool_mutex);
 		sprintf (tmpfd, "%d", fd);
+		g_print ("Removing sockfd: %s - %u\n", tmpfd, g_hash_table_size (client_thread_pool));
 		g_hash_table_foreach_remove (client_thread_pool, (GHRFunc)&is_fd_in_client_pool, tmpfd);
+		g_print ("Removed sockfd: %s - %u\n", tmpfd, g_hash_table_size (client_thread_pool));
 		pthread_mutex_unlock (&client_thread_pool_mutex);
-	}
+	} else
+		g_print ("Client thread pool not initalized\n");
 
 	close (fd);
 }
@@ -337,7 +349,7 @@ ldtp_server_thread (void *ptr)
 	int time_out = -1;
 	LDTPErrorCode err;
 
-	if (!ldtp_script_service && !ldtp_script_port) {
+	if (!ldtp_script_service) {
 		time_out = 1000 * 60 * 5; // Wait max of 5 minutes, if no clients then quit
 	}
 	server_socket = init_ldtp_server (LDTP_SCRIPT_SERVER);
@@ -409,6 +421,8 @@ main (int argc, char **argv)
 	char *env_sleep = NULL;
 	pthread_t server_thread = 0;
 	struct rlimit resource_limit;
+
+#ifdef ENABLE_GOPTIONPARSE
 	GOptionContext *context = NULL;
 
 	context = g_option_context_new ("- Linux Desktop Testing Project engine");
@@ -421,7 +435,94 @@ main (int argc, char **argv)
 		exit (1);
 	}
 	g_option_context_free (context);
+#else
+	int c;
 
+	while (1) {
+		static struct option long_options [] = {
+			/* These options set a flag. */
+			{"gui-timeout",  required_argument, 0, 'g'},
+			{"no-external-xml-file",  no_argument, 0, 'n'},
+			{"obj-timeout",  required_argument, 0, 'o'},
+			{"port",  required_argument, 0, 'p'},
+			{"script-engine", no_argument, &ldtp_script_service, 1},
+			{"help",     no_argument, 0, 'h'},
+			{"usage",     no_argument, 0, 'u'},
+			{"verbose",     no_argument, 0, 'v'},
+			{"version",     no_argument, 0, 'V'},
+			{0, 0, 0, 0}
+		};
+		/* getopt_long stores the option index here. */
+		int option_index = 0;
+
+		c = getopt_long (argc, argv, "g:o:p:hnsuvV",
+				 long_options, &option_index);
+     
+		/* Detect the end of the options. */
+		if (c == -1)
+			break;
+     
+		switch (c) {
+		case 'g':
+			ldtp_gui_timeout = atol (optarg);
+			if (ldtp_gui_timeout <= 0)
+				ldtp_gui_timeout = 30;
+			break;
+     
+		case 'o':
+			ldtp_obj_timeout = atol (optarg);
+			if (ldtp_obj_timeout <= 0)
+				ldtp_obj_timeout = 5;
+			break;
+     
+		case 'p':
+			ldtp_script_port = atol (optarg);
+			break;
+     
+		case 'h':
+			printf ("Usage:\n"
+"ldtp [OPTION...] - Linux Desktop Testing Project engine\n"
+"\n"
+"Help Options:\n"
+"	-h, --help              Show help options\n"
+"\n"
+"Application Options:\n"
+"	-g, --gui-timeout=N          Wait gui to appear / disappear till N seconds\n"
+"       -n, --no-external-xml-file   Don't use external temp XML file\n"
+"	-o, --obj-timeout=N          Wait object to appear till N seconds\n"
+"	-p, --port=N                 Start LDTP scripting engine on TCP port\n"
+"	-s, --script-engine          Start LDTP script execution engine as TCP service\n"
+"	-u, --usage                  LDTP engine usage\n"
+"	-v, --verbose                Verbose mode\n"
+"	-V, --version                LDTP engine version\n"
+"\n");
+			exit (0);
+     
+		case 'n':
+			ldtp_external_xml_file = FALSE;
+			break;
+     
+		case 's':
+			ldtp_script_service = TRUE;
+			break;
+     
+		case 'u':
+			ldtp_usage = TRUE;
+			break;
+     
+		case 'v':
+			ldtp_debug = TRUE;
+			break;
+     
+		case 'V':
+			ldtp_version = TRUE;
+			break;
+     
+		default:
+			ldtp_usage = TRUE;
+		}
+	}
+#endif
 	if (ldtp_usage) {
 		printf ("ldtp [--script-engine] [--port=N] [--verbose] [--gui-timeout=N]  [--obj-timeout=N] [--version] [--help]\n");
 		exit (0);
@@ -431,6 +532,9 @@ main (int argc, char **argv)
 		exit (0);
 	}
 
+	if (ldtp_script_port)
+		ldtp_script_service = TRUE;
+
 	/* generate core dump on seg-fault */
 	resource_limit.rlim_cur =
 		resource_limit.rlim_max = RLIM_INFINITY;
