@@ -54,7 +54,7 @@ select_menu_item (Accessible *object, FILE *log_fp)
 
 	if (wait_till_object_state_contains (object, MENU_ITEM, log_fp) != 0) {
 		LDTPErrorCode error;
-		error =  (LDTP_ERROR_INVALID_OBJECT_STATE);
+		error = LDTP_ERROR_INVALID_OBJECT_STATE;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -64,7 +64,7 @@ select_menu_item (Accessible *object, FILE *log_fp)
 		name = Accessible_getName (object);
 		g_print ("Unable to get Action handle: %s\n", name);
 		SPI_freeString (name);
-		error =  (LDTP_ERROR_SELECT_MENU_ITEM_FAILED);
+		error = LDTP_ERROR_SELECT_MENU_ITEM_FAILED;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
