@@ -65,7 +65,7 @@ select_menu_item (Accessible *object, char *param)
 	//  */
 	//  AccessibleAction_doAction (action, 0);
 	//  Accessible_unref (action);
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }
 
 static LDTPErrorCode
@@ -176,9 +176,9 @@ menu_main (LDTPClientContext* cctxt, int command, char *window_name)
 	case LDTP_CMD_VERIFYMENUCHECK:
 	case LDTP_CMD_VERIFYMENUUNCHECK:
 	case LDTP_CMD_DOESMENUITEMEXIST: {
-		char *token       = NULL;
-		char *rest_token  = NULL;
-		char *rest_params = NULL;
+		//char *token       = NULL;
+		//char *rest_token  = NULL;
+		//char *rest_params = NULL;
 
 		AccessibleRole class;
 		LDTPRequest* ldtp_req = NULL;
@@ -190,68 +190,108 @@ menu_main (LDTPClientContext* cctxt, int command, char *window_name)
 		if (cctxt && cctxt->req && cctxt->req->component)
 			g_print ("DEBUG: %s - %d - %s\n", __FILE__, __LINE__, cctxt->req->component);
 
+		ldtp_req = cctxt->req;
 		//params = mnuFile;mnuNew
 		if (cctxt->req && cctxt->req->arg_list && cctxt->req->arg_list->data) {
-			token = strtok (cctxt->req->arg_list->data, ";");
-		} else {
-			if (token)
-				g_sprintf (msg, "Unable to get gui handle %s %s %d",
-					   token, __FILE__, __LINE__);
+			gchar **token = NULL;
+			token = g_strsplit (ldtp_req->arg_list->data, ";", 2);
+			gchar *tmp = token [1];
+			ldtp_req->arg_list = g_slist_remove_all (ldtp_req->arg_list,
+								 ldtp_req->arg_list->data);
+			if (tmp) {
+				g_print ("Tmp: %s\n", tmp);
+				ldtp_req->arg_list = g_slist_append (ldtp_req->arg_list, 
+								     g_strdup (tmp));
+			}
+			g_free (ldtp_req->component);
+			g_print ("Token: %s\n", token [0]);
+			ldtp_req->component = g_strdup (token [0]);
+			g_strfreev (token);
+			if (ldtp_req->component)
+				accessible = ldtp_gui_get_gui_handle (cctxt, &error);
 			else
-				g_sprintf (msg, "Unable to get gui handle %s %d",
-					   __FILE__, __LINE__);
+				accessible = ldtp_gui_get_gui_handle (cctxt, &error);
+		} else {
+			//if (token)
+			//	g_sprintf (msg, "Unable to get gui handle %s %s %d",
+			//		   token, __FILE__, __LINE__);
+			//else
+			//	g_sprintf (msg, "Unable to get gui handle %s %d",
+			//		   __FILE__, __LINE__);
+			g_sprintf (msg, "Unable to get gui handle %s %d",
+				   __FILE__, __LINE__);
 			g_print ("%s\n", msg);
 			log_msg (LDTP_LOG_CAUSE, msg, cctxt->log_fp);
 			return LDTP_ERROR_UNABLE_TO_GET_CHILD_MENU_ITEM;
 		}
-		ldtp_req = cctxt->req;
-
-		if (token != NULL) {
-			g_print ("Component: %s - Token: %s\n", ldtp_req->component, token);
-			g_free (ldtp_req->component);
-			ldtp_req->component = g_strdup (token);
-			rest_token = strtok (NULL, ";");
-			while (rest_token) {
-				g_print ("Rest token: %s\n", rest_token);
-				if (!rest_params)
-					rest_params  = strdup (rest_token);
-				else {
-					rest_params = (char *) realloc (rest_params,
-									sizeof (char) *
-									(strlen (rest_params) +
-									 strlen (rest_token) + 1)
-									+ 1);
-					strcat (rest_params, ";");
-					strcat (rest_params, rest_token);
-				}
-				rest_token = strtok (NULL, ";");
-			}
-			/*
-			  Remove existing menu hierarchy from the list
-			*/
-			cctxt->req->arg_list = g_slist_remove_all (cctxt->req->arg_list,
-								   cctxt->req->arg_list->data);
-			if (rest_params) {
-				g_print ("Rest params: %s\n", rest_params);
-				/*
-				  Add rest of menu in the hierarchy to the list
-				*/
-				cctxt->req->arg_list = g_slist_append (cctxt->req->arg_list, rest_params);
-			}
-		}
-		if (token)
-			accessible = ldtp_gui_get_gui_handle (cctxt, &error);
-		else
-			accessible = ldtp_gui_get_gui_handle (cctxt, &error);
+		//if (cctxt->req && cctxt->req->arg_list && cctxt->req->arg_list->data) {
+		//	token = strtok (cctxt->req->arg_list->data, ";");
+		//} else {
+		//	if (token)
+		//		g_sprintf (msg, "Unable to get gui handle %s %s %d",
+		//			   token, __FILE__, __LINE__);
+		//	else
+		//		g_sprintf (msg, "Unable to get gui handle %s %d",
+		//			   __FILE__, __LINE__);
+		//	g_print ("%s\n", msg);
+		//	log_msg (LDTP_LOG_CAUSE, msg, cctxt->log_fp);
+		//	return LDTP_ERROR_UNABLE_TO_GET_CHILD_MENU_ITEM;
+		//}
+		//ldtp_req = cctxt->req;
+		//
+		//if (token != NULL) {
+		//	g_print ("Component: %s - Token: %s\n", ldtp_req->component, token);
+		//	g_free (ldtp_req->component);
+		//	ldtp_req->component = g_strdup (token);
+		//	rest_token = strtok (NULL, ";");
+		//	while (rest_token) {
+		//		g_print ("Rest token: %s\n", rest_token);
+		//		if (!rest_params)
+		//			rest_params  = strdup (rest_token);
+		//		else {
+		//			rest_params = (char *) realloc (rest_params,
+		//							sizeof (char) *
+		//							(strlen (rest_params) +
+		//							 strlen (rest_token) + 1)
+		//							+ 1);
+		//			strcat (rest_params, ";");
+		//			strcat (rest_params, rest_token);
+		//		}
+		//		rest_token = strtok (NULL, ";");
+		//	}
+		//	/*
+		//	  Remove existing menu hierarchy from the list
+		//	*/
+		//	cctxt->req->arg_list = g_slist_remove_all (cctxt->req->arg_list,
+		//						   cctxt->req->arg_list->data);
+		//	if (rest_params) {
+		//		g_print ("Rest params: %s\n", rest_params);
+		//		/*
+		//		  Add rest of menu in the hierarchy to the list
+		//		*/
+		//		cctxt->req->arg_list = g_slist_append (cctxt->req->arg_list, rest_params);
+		//	}
+		//}
+		//if (token)
+		//	accessible = ldtp_gui_get_gui_handle (cctxt, &error);
+		//else
+		//	accessible = ldtp_gui_get_gui_handle (cctxt, &error);
 					
 		if (!accessible) {
-			if (token)
-				g_sprintf (msg, "Unable to get gui handle %s %s %d",
-					   token, __FILE__, __LINE__);
-			else
+			//if (token)
+			//	g_sprintf (msg, "Unable to get gui handle %s %s %d",
+			//		   token, __FILE__, __LINE__);
+			//else
+			//	g_sprintf (msg, "Unable to get gui handle %s %s %d",
+			//		   (char *)cctxt->req->arg_list->data,
+			//		   __FILE__, __LINE__);
+			if (cctxt->req->arg_list && cctxt->req->arg_list->data)
 				g_sprintf (msg, "Unable to get gui handle %s %s %d",
 					   (char *)cctxt->req->arg_list->data,
 					   __FILE__, __LINE__);
+			else
+				g_sprintf (msg, "Unable to get gui handle %s %d",
+					   __FILE__, __LINE__);
 			g_print ("%s\n", msg);
 			log_msg (LDTP_LOG_CAUSE, msg, cctxt->log_fp);
 			return LDTP_ERROR_UNABLE_TO_GET_CHILD_MENU_ITEM;
@@ -272,7 +312,7 @@ menu_main (LDTPClientContext* cctxt, int command, char *window_name)
 			error = radio_menu_item_main (cctxt, command);
 		else if (class == SPI_ROLE_CHECK_MENU_ITEM)
 			error = check_menu_item_main (cctxt, command);
-		free (rest_params);
+		//free (rest_params);
 		Accessible_unref (accessible->handle);
 		g_free (accessible);
 		cctxt->gui_handle->handle = object;
@@ -286,7 +326,7 @@ menu_main (LDTPClientContext* cctxt, int command, char *window_name)
 		if (error == LDTP_ERROR_SUCCESS && cctxt->resp->data)
 			cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
 		if (cctxt->resp->data)
-			g_print ("\nSubmenus: %s - %o\n", cctxt->resp->data, cctxt->resp->data_len);
+			g_print ("\nSubmenus: %s - %ld\n", cctxt->resp->data, cctxt->resp->data_len);
 		break;
 	case LDTP_CMD_SELECTPOPUPMENU:
 		error = select_popup_menu (cctxt);
