@@ -152,18 +152,18 @@ page_tab_list_main (LDTPClientContext* cctxt, int command)
 	switch (command) {
 	case LDTP_CMD_SELECTTAB:
 		if (!cctxt->req->arg_list->data) {
-			error =  (LDTP_ERROR_PAGE_TAB_NAME_INPUT_DOESNOT_EXIST);
+			error = LDTP_ERROR_PAGE_TAB_NAME_INPUT_DOESNOT_EXIST;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 			return error;
 		}
-		g_print ("Tab name: %s\n", (char *)cctxt->req->arg_list);
+		g_print ("Tab name: %s\n", (char *)cctxt->req->arg_list->data);
 		error = select_tab (cctxt->gui_handle->handle,
 				    cctxt->req->arg_list->data, cctxt->log_fp);
 		break;
 	case LDTP_CMD_SELECTTABINDEX: {
 		long tab_index;
 		if (!cctxt->req->arg_list || !cctxt->req->arg_list->data) {
-			error =  (LDTP_ERROR_PAGE_TAB_INDEX_INPUT_DOESNOT_EXIST);
+			error = LDTP_ERROR_PAGE_TAB_INDEX_INPUT_DOESNOT_EXIST;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 			return error;
 		}
@@ -178,7 +178,7 @@ page_tab_list_main (LDTPClientContext* cctxt, int command)
 		error = panel_main (cctxt, LDTP_CMD_GETPANELCHILDCOUNT);
 		break;
 	default:
-		error =  (LDTP_ERROR_COMMAND_NOT_IMPLEMENTED);
+		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 		break;
 	}
 	return error;
