@@ -43,8 +43,10 @@ select_panel_name (Accessible *object, char *panel_name, FILE *log_fp)
 	AccessibleComponent *accessible_component = NULL;
 
 	name = Accessible_getName (object);
-	g_print ("Panel name: %s\n", name);
-	SPI_freeString (name);
+	if (name) {
+		g_print ("Panel name: %s\n", name);
+		SPI_freeString (name);
+	}
 
 	child_count = Accessible_getChildCount (object);
 	if (child_count == -1) {
@@ -54,15 +56,20 @@ select_panel_name (Accessible *object, char *panel_name, FILE *log_fp)
 	}
 	for (i = 0; i < child_count; i++) {
 		child_object = Accessible_getChildAtIndex (object, i);
-		name = Accessible_getName (child_object);
-		if (name && g_pattern_match_simple (panel_name, name)) {
-			accessible_component = Accessible_getComponent (child_object);
-			SPI_freeString (name);
-			break;
-		}  
-		SPI_freeString (name);
-		Accessible_unref (child_object);
-		child_object = NULL;
+		if (child_object) {
+			name = Accessible_getName (child_object);
+			if (name) {
+				g_print ("Panel name: %s - %s\n", name, panel_name);
+				if (g_pattern_match_simple (panel_name, name)) {
+					accessible_component = Accessible_getComponent (child_object);
+					SPI_freeString (name);
+					break;
+				}
+				SPI_freeString (name);
+			}
+			Accessible_unref (child_object);
+			child_object = NULL;
+		}
 	}
 	if (accessible_component) {
 		flag = AccessibleComponent_grabFocus (accessible_component);
@@ -167,7 +174,7 @@ panel_main (LDTPClientContext* cctxt, int command)
 		}
 		break;
 	default:
-		error =  (LDTP_ERROR_COMMAND_NOT_IMPLEMENTED);
+		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 		break;
 	}
 	return error;
