@@ -74,12 +74,10 @@ click (Accessible *object, FILE *log_fp)
 		flag = AccessibleComponent_grabFocus (accessible_component);
 		Accessible_unref (accessible_component);
 	}
-	/*
-	  Fixme: In file-roller Add dialog, add button action doesn't work. So manually add them
-	  for timebeing
-	*/
 	action = Accessible_getAction (object);
+	flag = FALSE;
 	if (action) {
+		sleep (1);
 		flag = AccessibleAction_doAction (action, 0);
 		Accessible_unref (action);
 	}
