@@ -233,7 +233,7 @@ get_keybinding (Accessible *object)
 	char *binding = NULL;
 	char *key_binding = NULL;
 	AccessibleAction *action = NULL;
-	if (!object || !Accessible_isAction (object))
+	if (object == NULL || !Accessible_isAction (object))
 		return NULL;
 
 	action = Accessible_getAction (object);
@@ -429,6 +429,11 @@ get_object_info (Accessible *accessible)
 		obj_info->instance_index = -1; /* Value -1 signifies not applicable */
 		obj_info->object_type = g_strdup ("font_chooser");
 	}
+	else if (role == SPI_ROLE_COLOR_CHOOSER) {
+		obj_info->prefix = g_strdup ("dlg");
+		obj_info->instance_index = -1; /* Value -1 signifies not applicable */
+		obj_info->object_type = g_strdup ("color_chooser");
+	}
 	else if (role == SPI_ROLE_RADIO_BUTTON) {
 		obj_info->prefix = g_strdup ("rbtn");
 		obj_info->instance_index = radio_button;
@@ -842,7 +847,8 @@ add_appmap_data (Accessible *accessible, char *parent_name, int child_index,
 		role = Accessible_getRole (accessible);
 		if (role == SPI_ROLE_FRAME || role == SPI_ROLE_DIALOG ||
 		    role == SPI_ROLE_ALERT || role == SPI_ROLE_FONT_CHOOSER ||
-		    role == SPI_ROLE_FILE_CHOOSER || role == SPI_ROLE_WINDOW)
+		    role == SPI_ROLE_FILE_CHOOSER || role == SPI_ROLE_WINDOW ||
+		    role == SPI_ROLE_COLOR_CHOOSER)
 			flag = TRUE;
 		if (g_utf8_strchr (label, -1, ' '))
 			stripped_data = escape_character (label, ' ');
@@ -1139,7 +1145,7 @@ get_window_name_in_appmap_format (char *window_name, AccessibleRole role)
 		type = g_strdup ("frm");
 	else if (role == SPI_ROLE_DIALOG || role == SPI_ROLE_ALERT ||
 		 role == SPI_ROLE_FONT_CHOOSER || role == SPI_ROLE_FILE_CHOOSER ||
-		 role == SPI_ROLE_WINDOW)
+		 role == SPI_ROLE_WINDOW || role == SPI_ROLE_COLOR_CHOOSER)
 		type = g_strdup ("dlg");
 	else
 		type = g_strdup ("");
