@@ -885,10 +885,17 @@ right_click (Accessible *object, GSList *l, FILE *log_fp)
 	Accessible *cell;
 	AccessibleTable *table;
 	char *name = NULL;
-	char *row_text = l->data;
+	char *row_text = NULL;
 	long n_matches = 0, matches = 0;
 	long x, y, height, width;
 
+	if (l && l->data)
+		row_text = l->data;
+	else {
+		error =  (LDTP_ERROR_RIGHT_CLICK_FAILED);
+		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
+		return error;
+	}
 	l = l->next;
 	if (l && l->data)
 		n_matches = atol (l->data);
@@ -1559,7 +1566,7 @@ tree_table_main (LDTPClientContext* cctxt, int command)
 		long col = 0;
 		if (cctxt->req->arg_list && cctxt->req->arg_list->data)
 			row = atol (cctxt->req->arg_list->data);
-		/* 
+		/* FIXME:
 		 * currently expand table cell performs 
 		 * action on the first column only 
 		 */
